<?php

namespace Drupal\content_connected;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Link;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\node\NodeInterface;

/**
 * Implements Class ContentConnectedManager.
 *
 * @package Drupal\content_connected
 */
class ContentConnectedManager implements ContentConnectedManagerInterface {

  use StringTranslationTrait;

  use DependencySerializationTrait {
    __wakeup as defaultWakeup;
    __sleep as defaultSleep;
  }

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Forum settings config object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Drupal\Core\Extension\ModuleHandler definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Constructs the content connected manager service.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The entity manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The render service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The translation manager service.
   */
  public function __construct(EntityTypeManager $entity_type_manager, ConfigFactoryInterface $config_factory, ModuleHandler $module_handler, RendererInterface $renderer, TranslationInterface $string_translation) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->moduleHandler = $module_handler;
    $this->renderer = $renderer;
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityRefrenceFields() {

    $fields = [];
    $link_fields = $this->entityTypeManager->getStorage('field_storage_config')->loadByProperties(
      [
        'settings' => [
          'target_type' => 'node',
        ],
        'entity_type' => 'node',
        'type' => 'entity_reference',
        'deleted' => FALSE,
        'status' => 1,
      ]
    );
    foreach ($link_fields as $field) {
      $fields[$field->getName()] = $field->getLabel();
    }
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getlinkFields() {
    $fields = [];
    $link_fields = $this->entityManager->getStorage('field_storage_config')->loadByProperties(
      [
        'entity_type' => 'node',
        'type' => 'link',
        'deleted' => FALSE,
        'status' => 1,
      ]
    );
    foreach ($link_fields as $field) {
      $fields[$field->getName()] = $field->getLabel();
    }
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getLongTextFields() {
    $fields = [];
    $text_with_summary = $this->entityTypeManager->getStorage('field_storage_config')->loadByProperties(
      [
        'entity_type' => 'node',
        'type' => 'text_with_summary',
        'deleted' => FALSE,
        'status' => 1,
      ]
    );
    $text_long = $this->entityTypeManager->getStorage('field_storage_config')->loadByProperties(
      [
        'entity_type' => 'node',
        'type' => 'text_long',
        'deleted' => FALSE,
        'status' => 1,
      ]
    );

    $long_fields = array_merge($text_with_summary, $text_long);
    foreach ($long_fields as $field) {
      $fields[$field->getName()] = $field->getLabel();
    }
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  protected function matchesEntityRefrenceField($nid) {

    $matches = [];
    $fields = $this->getEntityRefrenceFields();

    $config = $this->configFactory->get('content_connected.adminsettings');
    $exclude_fields = $config->get('content_connected_exclude_entityreffields');

    $fields = $this->excludeFields($fields, $exclude_fields);

    foreach ($fields as $fieldname => $field_value) {
      $matches[$fieldname] = $this->entityTypeManager->getStorage('node')->getQuery()
          ->condition('status', 1)
          ->accessCheck()
          ->condition($fieldname, $nid)->execute();
    }

    return ['entity_reference' => $matches];
  }

  /**
   * {@inheritdoc}
   */
  protected function matchesLinkField($nid) {
    $matches = [];

    $fields = $this->getlinkFields();
    $node = $this->entityTypeManager->getStorage('node')->load($nid);
    $config = $this->configFactory->get('content_connected.adminsettings');
    $exclude_fields = $config->get('content_connected_exclude_linkfields');

    $fields = $this->excludeFields($fields, $exclude_fields);

    foreach ($fields as $fieldname => $field_value) {

      $matches[$fieldname] = $this->entityTypeManager->getStorage('node')->getQuery()
          ->condition('status', 1)
          ->accessCheck()
          ->condition($fieldname, 'internal:/node/' . $node->id() , 'LIKE' )->execute();
    }
    return ['Link_field' => $matches];
  }

  /**
   * {@inheritdoc}
   */
  protected function matchesLongTextField($nid) {
    $matches = [];
    $fields = $this->getLongTextFields();

    $node = $this->entityTypeManager->getStorage('node')->load($nid);
    $config = $this->configFactory->get('content_connected.adminsettings');
    $exclude_fields = $config->get('content_connected_exclude_longtextfields');

    $fields = $this->excludeFields($fields, $exclude_fields);

    foreach ($fields as $fieldname => $field_value) {
      $matches[$fieldname] = $this->entityTypeManager->getStorage('node')->getQuery()
          ->condition('status', 1)
          ->accessCheck()
          ->condition($fieldname, 'node/' . $node->id(), 'CONTAINS')->execute();
    }
    return ['long_text' => $matches];
  }

  /**
   * Render function to generate content connected table.
   */
  public function renderMatches($nid) {

    $matches = array_merge($this->matchesEntityRefrenceField($nid), $this->matchesLinkField($nid), $this->matchesLongTextField($nid));

    // Allow other modules to alter the reference array.
    $this->moduleHandler->alter('content_connected', $nid, $matches);
    $headers = [
      $this->t('Entity type'),
      $this->t('Title'),
      $this->t('Content type'),
      $this->t('Status'),
      $this->t('Type of field'),
    ];
    $rows = [];
    foreach ($matches as $match_type => $match) {
      foreach ($match as $field_name => $ids) {
        $entity_type_id = 'node';
        if (strpos($field_name, ':') !== FALSE) {
          [$entity_type_id, $field_name] = explode(':', $field_name);
        }

        $entities = $this->entityTypeManager->getStorage($entity_type_id)->loadMultiple($ids);

        foreach ($entities as $entity) {
          $row = [];
          $row[] = $entity->getEntityType()->getLabel();
          $row[] = Link::fromTextAndUrl($entity->label(), $entity->toUrl());
          $row[] = $entity->getEntityType()->getBundleLabel();
          $row[] = method_exists($entity, 'isPublished') ? (
            $entity->isPublished() == \Drupal\node\NodeInterface::PUBLISHED
              ? $this->t('Published') : $this->t('Not published')
          ) : $this->t('n/a');
          $row[] = $match_type . '(' . $field_name . ')';
          $rows[] = $row;
        }
      }
    }

    $build = [
      '#theme' => 'table',
      '#caption' => $this->t('Content connected'),
      '#header' => $headers,
      '#rows' => $rows,
      '#empty' => $this->t('No content connected available.'),
    ];

    return $this->renderer->render($build);
  }

  /**
   * Helper function to exclude fields.
   */
  protected function excludeFields($fields, $exclude_fields) {

    if (!empty($exclude_fields)) {
      $fields = array_diff_key($fields, array_filter($exclude_fields));
    }
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function hasContentDependant($nid): bool {
    $hasItems = function ($matches) {
      foreach ($matches as $match_name => $match_array) {
        foreach ($match_array as $match) {
          if (count($match)>0){
            return TRUE;
          }
        }
      }
      return FALSE;
    };

    $matches = $this->matchesEntityRefrenceField($nid);
    if ($hasItems($matches)) {
      return TRUE;
    }

    $matches =  $this->matchesLinkField($nid);
    if ($hasItems($matches)){
      return TRUE;
    }

    $matches = $this->matchesLongTextField($nid);
    if ($hasItems($matches)){
      return TRUE;
    }

    $matches = [];
    $this->moduleHandler->alter('content_connected', $nid, $matches);
    if ($hasItems($matches)){
      return TRUE;
    }

    return FALSE;
  }

}
